#include <glib.h>
#include <glib/gi18n.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "libbox-db.h"
#include "libbox-db-utils.h"
#include "libbox-db-deps.h"
#include "libbox-db-repos.h"
#include "libbox-parser.h"
#include "libbox-repos.h"

int main (int argc, char *argv[])
{
    DepsResolve *deps;
    GList *li;
    sqlite3 *db;
    gchar *name;
    int rc;
    sqlite3_stmt *stmt = NULL;
    gchar *sql = NULL;

    db = box_db_open("/");

    box_db_attach_repos(db, "/");
    box_db_repos_init(db);

    sql = g_strdup("SELECT name FROM u_package ORDER BY name");

    rc = sqlite3_prepare(db, sql, strlen(sql), &stmt, NULL);

    while ( (rc = sqlite3_step(stmt)) == SQLITE_ROW )
    {
        if (rc == SQLITE_ROW)
        {
            /*printf(":%s:\n", (gchar *)sqlite3_column_text(stmt, 0)); */
            name = (gchar *)sqlite3_column_text(stmt, 0);
            deps = box_db_repos_resolve_deps_one(db, name);

            for (li = deps->deps; li != NULL; li = li->next)
            {
                printf("%s\t%s\n", name, (gchar*)li->data);
            }

            for (li = deps->missing; li != NULL; li = li->next)
            {
                fprintf(stderr, "%s\t%s\n", name, (gchar*)li->data);
            }
        } else if (rc != SQLITE_OK)
        {
            printf("ERROR: %s\n", sqlite3_errmsg(db));
        }
    }
    sqlite3_finalize(stmt);

    g_free(sql);

    box_db_close(db);

    return 0;
}
