#include <glib.h>
#include <glib/gi18n.h>
#include <stdio.h>

#include "libbox-parser.h"

static gboolean pkg_name = FALSE;
static gboolean pkg_version = FALSE;
static gboolean pkg_release = FALSE;
static gboolean pkg_group = FALSE;
static gchar* filename = NULL;

static GOptionEntry entries[] = 
{
    { "pkg-name", 'n', 0, G_OPTION_ARG_NONE, &pkg_name, "Show package name", NULL },
    { "pkg-version", 'v', 0, G_OPTION_ARG_NONE, &pkg_version, "Show package version", NULL },
    { "pkg-release", 'r', 0, G_OPTION_ARG_NONE, &pkg_release, "Show package release", NULL },
    { "pkg-group", 'g', 0, G_OPTION_ARG_NONE, &pkg_group, "Show package group", NULL },
    { "filename", 'f', 0, G_OPTION_ARG_STRING, &filename, "box filename", NULL },
    { NULL }
};

int main (int argc, char *argv[])
{
    GError *error = NULL;
    GOptionContext *context;
    BoxDocument *document = NULL;

    context = g_option_context_new ("box-info");
    g_option_context_add_main_entries (context, entries, NULL);
    g_option_context_parse (context, &argc, &argv, &error);


    if (filename == NULL)
        g_error("Please specify filename");


    document =  box_document_new(filename);

    if (document != NULL)
    {
        if (pkg_name)
            printf("%s\n", document->header->name);
        if (pkg_version)
            printf("%s\n", document->header->version);
        if (pkg_release)
            printf("%s\n", document->header->release);
        if (pkg_group)
            printf("%s\n", document->header->group);

        box_document_free(document);
    } else
        g_error("Header is NULL");

    g_option_context_free(context);

    return 0;
}
