#include <glib.h>
#include <glib/gi18n.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "libbox.h"
#include "libbox-db.h"
#include "libbox-db-deps.h"
#include "libbox-db-utils.h"
#include "libbox-db-repos.h"
#include "libbox-parser.h"
#include "libinternal-utils.h"

static gchar* arg_search = NULL;
static gchar* arg_install = NULL;
static gchar* arg_what_provide = NULL;
static gchar* arg_what_require = NULL;
static gchar* dest_dir = "/";

gboolean arg_search_cb (const char *key, const char *value, gpointer user_data)
{
    if (value == NULL)
        arg_search = g_strdup("");
    else
        arg_search = g_strdup((gchar *)value);

    return TRUE;
}

static GOptionEntry entries[] = 
{
    { "search", 's', G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, arg_search_cb, "Search for a new packages to install", "PKG" },
    { "install", 'i', 0, G_OPTION_ARG_STRING, &arg_install, "Show path to package(s) for installation", "PKG" },
    { "root", 0, 0, G_OPTION_ARG_STRING, &dest_dir, "Specify installation directory (default is '/')", "PATH" },
    { "what-provide", 0, 0, G_OPTION_ARG_STRING, &arg_what_provide, "Show package which provide X", "X" },
    { "what-require", 0, 0, G_OPTION_ARG_STRING, &arg_what_require, "Show package which require X", "X" },
    { NULL }
};

static void print_list_str(GList *list)
{
    GList *li = NULL;
    PackageSearch *search = NULL;

    for (li = list; li != NULL; li = li->next)
    {
        search = (PackageSearch *)li->data;
        printf("%s\n", search->package);
    }
}

static void print_list_str_location(GList *list)
{
    GList *li = NULL;
    PackageSearch *search = NULL;

    for (li = list; li != NULL; li = li->next)
    {
        search = (PackageSearch *)li->data;
        printf("%s\n", search->location);
    }
}

int main (int argc, char *argv[])
{
    GError *error = NULL;
    GOptionContext *context;

    GList *list = NULL;
    sqlite3 *db = NULL;

    context = g_option_context_new ("box-repos");
    g_option_context_add_main_entries (context, entries, NULL);
    g_option_context_set_help_enabled(context, TRUE);
    g_option_group_set_error_hook(g_option_context_get_main_group(context), (GOptionErrorFunc)goption_error_func);
    g_option_context_parse (context, &argc, &argv, &error);


    db = box_db_open(dest_dir);

    box_db_create_empty(db);
    box_db_attach_repos(db, box_get_repos_dir(dest_dir));
    box_db_repos_init(db);


    if ((arg_search) || (arg_install) || (arg_what_provide) || (arg_what_require))
    {
        if (arg_search)
        {
            list = box_db_repos_packages_search_available(db, arg_search, PKG_DEVEL | PKG_NON_DEVEL);
            print_list_str(list);
            box_db_repos_package_list_free(list);
            g_free(arg_search);
        } else if (arg_install)
        {
            list = box_db_repos_package_get_for_installation(db, arg_install);
            print_list_str_location(list);
            box_db_repos_package_list_free(list);
        } else if (arg_what_provide)
        {
            list = box_db_repos_what_provide(db, arg_what_provide);
            print_list_str(list);
            box_db_repos_package_list_free(list);
        } else if (arg_what_require)
        {
            list = box_db_repos_what_require(db, arg_what_require);
            print_list_str(list);
            box_db_repos_package_list_free(list);
        }
    } else {
        g_print("Wrong parameters.\n");
        goption_show_error(context);
    }

    g_option_context_free(context);

    box_db_close(db);

    return 0;
}
