#include <glib.h>
#include <glib/gi18n.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "libbox-db.h"
#include "libbox-db-deps.h"
#include "libbox-db-utils.h"
#include "libbox-db-repos.h"
#include "libbox-repos.h"
#include "libbox-parser.h"
#include "libbox.h"
#include "libinternal-utils.h"

static gchar* arg_uninstall = NULL;
static gchar* arg_install_file = NULL;
static gchar* arg_install_factory = NULL;
static gboolean arg_upgrade_dist = FALSE;
static gboolean arg_sync_repos = FALSE;
static gboolean arg_no_deps = FALSE;
static gchar* arg_run_scripts = NULL;
static gchar* dest_dir = "/";


static GOptionEntry entries[] = 
{
    { "install", 'i', 0, G_OPTION_ARG_STRING, &arg_install_file, "Install package by file", NULL },
    { "install-factory", 0, 0, G_OPTION_ARG_STRING, &arg_install_factory, "Install package by file to the factory", NULL },
    { "uninstall", 'u', 0, G_OPTION_ARG_STRING, &arg_uninstall, "Uninstall package by name", NULL },
    { "upgrade-dist", 'x', 0, G_OPTION_ARG_NONE, &arg_upgrade_dist, "Upgrade distribution", NULL },
    { "sync-repos", 's', 0, G_OPTION_ARG_NONE, &arg_sync_repos, "Sync repos", NULL },
    { "root", 0, 0, G_OPTION_ARG_STRING, &dest_dir, "Installation dir, defaults /", NULL },
    { "no-deps", 'n', 0, G_OPTION_ARG_NONE, &arg_no_deps, "Skip dependencies checking for uninstall", NULL },
    { "run-scripts", 0, 0, G_OPTION_ARG_STRING, &arg_run_scripts, "Run scripts <pre-install|post-install>", NULL },
    { NULL }
};

static void uninstall_progress(int value, void *data)
{
/*    printf("%d%%\n", value); */
}

int main (int argc, char *argv[])
{
    GError *error = NULL;
    GOptionContext *context;
    gint result = 0;
    gint script_type = -1;

    context = g_option_context_new ("box");
    g_option_context_add_main_entries (context, entries, NULL);
    g_option_context_set_help_enabled(context, TRUE);
    g_option_group_set_error_hook(g_option_context_get_main_group(context), (GOptionErrorFunc)goption_error_func);
    g_option_context_parse (context, &argc, &argv, &error);

    if (arg_uninstall)
    {
        result = !box_package_uninstall(arg_uninstall, dest_dir, uninstall_progress, NULL, arg_no_deps);
    } else if (arg_install_file)
    {
        result = !box_package_install(arg_install_file, dest_dir, uninstall_progress, NULL, arg_no_deps);
    } else if (arg_install_factory)
    {
        result = !box_package_install_factory(arg_install_factory, dest_dir, uninstall_progress, NULL);
    } else if (arg_upgrade_dist)
    {
        result = !box_upgrade_dist(dest_dir, uninstall_progress, NULL);
    } else if (arg_sync_repos)
    {
        box_repos_sync(dest_dir, NULL, NULL);
    } else if (arg_run_scripts)
    {
        if (strcmp(arg_run_scripts, "pre-install") == 0)
        {
            script_type = SCRIPT_TYPE_PRE_INSTALL;
        } else if (strcmp(arg_run_scripts, "post-install") == 0)
        {
            script_type = SCRIPT_TYPE_POST_INSTALL;
        } else
        {
            g_print("Wrong script type.\n");
            goption_show_error(context);
        }
        if (script_type >= 0)
        {
            box_run_scripts(dest_dir, script_type, TRUE);
        }
    } else
    {
        g_print("Wrong parameters.\n");
        goption_show_error(context);
    }

    g_option_context_free(context);

    return result;
}
