#ifndef _DB_H_
#define _DB_H_

#include "config.h"
#include <glib.h>
#include <sqlite3.h>

#define sqlite3_prepare sqlite3_prepare_v2

typedef struct {
    gchar *file;
    gchar *package;
} SearchResult;

typedef struct {
    gchar *name;
    gchar *version;
    gchar *release;
    gchar *inst_date;
} PackageInfo;

enum SCRIPT_TYPE {
    SCRIPT_TYPE_PRE_INSTALL = 0,
    SCRIPT_TYPE_POST_INSTALL
};

void box_db_file_info(const gchar *path, const gchar *path_relative);

gchar* box_db_get_file_path_from_id(sqlite3 *db, gint file_id, const gchar *db_prefix);
GList* box_db_get_file_list(sqlite3 *db, const gchar *name);
void box_db_file_list_free(GList *list);

GList* box_db_install(sqlite3 *db, const gchar *path, const gchar *path_relative, const gchar *name, const gchar *version, gint release, const gchar *summary);
GList* box_db_uninstall(sqlite3 *db, const gchar *name, gboolean force, GList **pkg_deps, gboolean *exist);

GList* box_db_search_file(sqlite3 *db, const gchar *name);

PackageInfo* box_db_package_info_new(const gchar *name, const gchar *version, const gchar *release,  const gchar *inst_date);
void box_db_package_info_free(PackageInfo *package);
GList* box_db_get_installed_packages(sqlite3 *db);
GList* box_db_search_dup_files(sqlite3 *db);

gint box_db_get_package_id(sqlite3 *db, const gchar* package);

void box_db_search_result_free(SearchResult *result);
void box_db_search_result_list_free(GList *list);

void box_db_install_script(sqlite3 *db, gint package_id, const gchar *script, enum SCRIPT_TYPE type, gint status);
GList* box_db_get_scripts(sqlite3 *db, enum SCRIPT_TYPE script_type, gboolean run_new);

#endif
